==> dynatrace_flutter_plugin-1.205.0.tar.gz: android/src/main/java/com/dynatrace/android/agent/DynatraceFlutterPlugin.java <==
package com.dynatrace.android.agent;

import android.location.Location;
import android.content.Context;
import android.util.Log;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.LinkedList;

import com.dynatrace.android.agent.conf.ConfigurationFactory;
import com.dynatrace.android.agent.conf.DataCollectionLevel;
import com.dynatrace.android.agent.conf.DynatraceConfigurationBuilder;
import com.dynatrace.android.agent.crash.PlatformType;

/**
 * Android implementation for Flutter plugin
 */
public class DynatraceFlutterPlugin implements MethodCallHandler {
  private Hashtable<Integer, DTXAction> actions;
  private Hashtable<Integer, WebRequestTiming> webTimings;
  private LinkedList<Integer> rootActions;

  private static final int PLATFORM_ANDROID = 0;
  private static final int DATA_COLLECTION_OFF = 0;

  private static int webTimingsCounter = 0;
  private Context applicationContext;

  /**
   * Plugin registration.
   */
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "dynatrace_flutter_plugin/dynatrace");
    channel.setMethodCallHandler(new DynatraceFlutterPlugin(registrar.activeContext()));
  }

  private DynatraceFlutterPlugin(Context ctx) {
    applicationContext = ctx;
    actions = new Hashtable<>();
    webTimings = new Hashtable<>();
    rootActions = new LinkedList<>();
    webTimingsCounter = 0;
  }

  public void onMethodCall(MethodCall call, Result result) {
    if (call.method.equals("enterAction")) {
      if (call.argument("parent") != null) {
        enterActionWithParent((String) call.argument("name"), (int) call.argument("key"), (int) call.argument("parent"),
            call.argument("platform"));
      } else {
        enterAction((String) call.argument("name"), (int) call.argument("key"), call.argument("platform"));
      }
    } else if (call.method.equals("leaveAction")) {
      leaveAction((int) call.argument("key"));
    } else if (call.method.equals("endVisit")) {
      endVisit(call.argument("platform"));
    } else if (call.method.equals("reportError")) {
      reportError((String) call.argument("errorName"), (int) call.argument("errorCode"), call.argument("platform"));
    } else if (call.method.equals("reportErrorWithStacktrace")) {
      reportErrorWithStacktrace((String) call.argument("errorName"), (String) call.argument("errorValue"),
          (String) call.argument("reason"), (String) call.argument("stacktrace"), call.argument("platform"));
    } else if (call.method.equals("reportCrash")) {
      reportCrash((String) call.argument("errorValue"), (String) call.argument("reason"),
          (String) call.argument("stacktrace"), call.argument("platform"));
    } else if (call.method.equals("reportErrorInAction")) {
      reportErrorInAction((int) call.argument("key"), (String) call.argument("errorName"),
          (int) call.argument("errorCode"), call.argument("platform"));
    } else if (call.method.equals("identifyUser")) {
      identifyUser((String) call.argument("user"), call.argument("platform"));
    } else if (call.method.equals("reportEventInAction")) {
      reportEventInAction((int) call.argument("key"), (String) call.argument("name"), call.argument("platform"));
    } else if (call.method.equals("reportStringValueInAction")) {
      reportStringValueInAction((int) call.argument("key"), (String) call.argument("name"),
          (String) call.argument("value"), call.argument("platform"));
    } else if (call.method.equals("reportIntValueInAction")) {
      reportIntValueInAction((int) call.argument("key"), (String) call.argument("name"), (int) call.argument("value"),
          call.argument("platform"));
    } else if (call.method.equals("reportDoubleValueInAction")) {
      reportDoubleValueInAction((int) call.argument("key"), (String) call.argument("name"),
          (double) call.argument("value"), call.argument("platform"));
    } else if (call.method.equals("setGPSLocation")) {
      setGPSLocation((double) call.argument("latitude"), (double) call.argument("longitude"),
          call.argument("platform"));
    } else if (call.method.equals("flushEvents")) {
      flushEvents(call.argument("platform"));
    } else if (call.method.equals("isCrashReportingOptedIn")) {
      result.success(isCrashReportingOptedIn(call.argument("platform")));
    } else if (call.method.equals("setCrashReportingOptedIn")) {
      setCrashReportingOptedIn((boolean) call.argument("crashReporting"), call.argument("platform"));
    } else if (call.method.equals("setDataCollectionLevel")) {
      setDataCollectionLevel((int) call.argument("dataCollectionLevel"), call.argument("platform"));
    } else if (call.method.equals("getDataCollectionLevel")) {
      result.success(getDataCollectionLevel(call.argument("platform")));
    } else if (call.method.equals("getRequestTag")) {
      result.success(getRequestTag());
    } else if (call.method.equals("startWebRequestTiming")) {
      result.success(startWebRequestTiming((String) call.argument("requestTag")));
    } else if (call.method.equals("stopWebRequestTiming")) {
      result.success(stopWebRequestTiming((int) call.argument("webTimingKey"), (String) call.argument("url"),
          (int) call.argument("responseCode"), (String) call.argument("responseMessage")));
    } else if (call.method.equals("start")) {
      start((String) call.argument("beaconUrl"), (String) call.argument("applicationId"),
          (Boolean) call.argument("userOptIn"), (Boolean) call.argument("crashReporting"),
          (String) call.argument("logLevel"), (Boolean) call.argument("certificateValidation"));
    } else if (call.method.equals("getAutoStartConfiguration")) {
      HashMap<String, Boolean> configuration = new HashMap<>();
      configuration.put("autoStart", ConfigurationFactory.getConfiguration().autoStart);
      configuration.put("logLevel", ConfigurationFactory.getConfiguration().debugLogLevel);
      configuration.put("webRequest", ConfigurationFactory.getConfiguration().webRequestTiming);
      configuration.put("crashReporting", ConfigurationFactory.getConfiguration().crashReporting);
      result.success(configuration);
    } else {
      result.notImplemented();
    }
  }

  private void start(String beaconUrl, String applicationId, Boolean userOptIn, Boolean crashReporting, String logLevel,
      Boolean certificateValidation) {
    DynatraceConfigurationBuilder builder = new DynatraceConfigurationBuilder(applicationId, beaconUrl);

    if (userOptIn != null) {
      builder.withUserOptIn(userOptIn);
    }

    if (crashReporting != null) {
      builder.withCrashReporting(crashReporting);
    }

    if (logLevel != null && logLevel.equals("debug")) {
      builder.withDebugLogging(true);
    }

    if (certificateValidation != null) {
      builder.withCertificateValidation(certificateValidation);
    }

    Dynatrace.startup(applicationContext, builder.buildConfiguration());
  }

  private void enterAction(String name, int key, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      DTXAction action = Dynatrace.enterAction(name);
      actions.put(key, action);
      rootActions.add(key);
    }
  }

  private void enterActionWithParent(String name, int key, int parentKey, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      DTXAction parent = actions.get(parentKey);

      if (parent != null) {
        DTXAction action = Dynatrace.enterAction(name, parent);
        actions.put(key, action);
      } else {
        enterAction(name, key, platform);
      }
    }
  }

  private void leaveAction(int key) {
    DTXAction action = actions.get(key);

    if (action != null) {
      if (action instanceof DTXAutoAction) {
        ((DTXAutoAction) action).startTimer();
      } else {
        action.leaveAction();
      }

      actions.remove(key);
    }

    rootActions.remove((Integer) key);
  }

  private void endVisit(Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Dynatrace.endVisit();
    }
  }

  private void reportError(String errorName, int errorCode, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Dynatrace.reportError(errorName, errorCode);
    }
  }

  /**
   * Reports a stacktrace - Used primary for the internal error handler
   * 
   * @param errorName  Non or Fatal React Native Error
   * @param errorValue Name of the Error - SyntaxError
   * @param reason     Reason for the Error
   * @param stacktrace Whole Stacktrace
   * @param platform   Platform wise or both
   */
  private void reportErrorWithStacktrace(String errorName, String errorValue, String reason, String stacktrace,
      Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Dynatrace.reportError(PlatformType.DART, errorName, errorValue, reason, stacktrace);
    }
  }

  public void reportCrash(String errorValue, String reason, String stacktrace, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Dynatrace.reportCrash(PlatformType.DART, errorValue, reason, stacktrace);
      Dynatrace.createNewSession();
    }
  }

  private void reportErrorInAction(int key, String errorName, int errorCode, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      DTXAction action = actions.get(key);
      if (action == null)
        return;
      action.reportError(errorName, errorCode);
    }
  }

  private void identifyUser(String user, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Dynatrace.identifyUser(user);
    }
  }

  private void reportEventInAction(int key, String name, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      DTXAction action = actions.get(key);
      if (action == null)
        return;
      action.reportEvent(name);
    }
  }

  private void reportStringValueInAction(int key, String name, String value, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      DTXAction action = actions.get(key);
      if (action == null)
        return;
      action.reportValue(name, value);
    }
  }

  private void reportIntValueInAction(int key, String name, int value, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      DTXAction action = actions.get(key);
      if (action == null)
        return;
      action.reportValue(name, value);
    }
  }

  private void reportDoubleValueInAction(int key, String name, double value, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      DTXAction action = actions.get(key);
      if (action == null)
        return;
      action.reportValue(name, value);
    }
  }

  private void setGPSLocation(double latitude, double longitude, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Location location = new Location("");
      location.setLatitude(latitude);
      location.setLongitude(longitude);
      Dynatrace.setGpsLocation(location);
    }
  }

  private void flushEvents(Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Dynatrace.flushEvents();
    }
  }

  private boolean isCrashReportingOptedIn(Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      return Dynatrace.isCrashReportingOptedIn();
    } else {
      return false;
    }
  }

  private void setCrashReportingOptedIn(boolean crashReporting, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Dynatrace.setCrashReportingOptedIn(crashReporting);
    }
  }

  private void setDataCollectionLevel(int collectionLevel, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Dynatrace.setDataCollectionLevel(DataCollectionLevel.values()[collectionLevel]);
    }
  }

  private int getDataCollectionLevel(Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      return Dynatrace.getDataCollectionLevel().ordinal();
    } else {
      return DATA_COLLECTION_OFF;
    }
  }

  private String getRequestTag() {
    if (!rootActions.isEmpty()) {
      DTXAction action = actions.get(rootActions.getFirst());

      if (action != null) {
        return action.getRequestTag();
      }
    }

    return Dynatrace.getRequestTag();
  }

  private int startWebRequestTiming(String requestTag) {
    if (requestTag != null) {
      WebRequestTiming timing = Dynatrace.getWebRequestTiming(requestTag);
      webTimings.put(webTimingsCounter++, timing);
      timing.startWebRequestTiming();
      return webTimingsCounter - 1;
    }

    return -1;
  }

  private Boolean stopWebRequestTiming(int webTimingKey, String url, int responseCode, String responseMessage) {
    if (webTimingKey != -1) {
      WebRequestTiming timing = webTimings.remove(webTimingKey);

      if (timing != null) {
        try {
          timing.stopWebRequestTiming(url, responseCode, responseMessage);
          return true;
        } catch (MalformedURLException ex) {
          return false;
        }
      }
    }

    return false;
  }

  private Boolean shouldWorkOnAndroid(Object platform) {
    return platform == null || ((int) platform) == PLATFORM_ANDROID;
  }
}

==> dynatrace_flutter_plugin-1.208.0.tar.gz: android/src/main/java/com/dynatrace/android/agent/DynatraceFlutterPlugin.java <==
package com.dynatrace.android.agent;

import android.location.Location;
import android.content.Context;
import android.util.Log;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.LinkedList;

import com.dynatrace.android.agent.conf.ConfigurationFactory;
import com.dynatrace.android.agent.conf.DataCollectionLevel;
import com.dynatrace.android.agent.conf.DynatraceConfigurationBuilder;
import com.dynatrace.android.agent.conf.UserPrivacyOptions;
import com.dynatrace.android.agent.crash.PlatformType;

/**
 * Android implementation for Flutter plugin
 */
public class DynatraceFlutterPlugin implements MethodCallHandler {
  private Hashtable<Integer, DTXAction> actions;
  // x-dynatrace header value (will always be unique) and the timing object associated with
  private Hashtable<String, WebRequestTiming> webTimings;
  private LinkedList<Integer> rootActions;

  private static final int PLATFORM_ANDROID = 0;
  private static final int DATA_COLLECTION_OFF = 0;

  private Context applicationContext;

  /**
   * Plugin registration.
   */
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "dynatrace_flutter_plugin/dynatrace");
    channel.setMethodCallHandler(new DynatraceFlutterPlugin(registrar.activeContext()));
  }

  private DynatraceFlutterPlugin(Context ctx) {
    applicationContext = ctx;
    actions = new Hashtable<>();
    webTimings = new Hashtable<>();
    rootActions = new LinkedList<>();
  }

  public void onMethodCall(MethodCall call, Result result) {
    if (call.method.equals("enterAction")) {
      if (call.argument("parent") != null) {
        enterActionWithParent((String) call.argument("name"), (int) call.argument("key"), (int) call.argument("parent"),
            call.argument("platform"));
      } else {
        enterAction((String) call.argument("name"), (int) call.argument("key"), call.argument("platform"));
      }
    } else if (call.method.equals("leaveAction")) {
      leaveAction((int) call.argument("key"));
    } else if (call.method.equals("endVisit")) {
      endVisit(call.argument("platform"));
    } else if (call.method.equals("reportError")) {
      reportError((String) call.argument("errorName"), (int) call.argument("errorCode"), call.argument("platform"));
    } else if (call.method.equals("reportErrorWithStacktrace")) {
      reportErrorWithStacktrace((String) call.argument("errorName"), (String) call.argument("errorValue"),
          (String) call.argument("reason"), (String) call.argument("stacktrace"), call.argument("platform"));
    } else if (call.method.equals("reportCrash")) {
      reportCrash((String) call.argument("errorValue"), (String) call.argument("reason"),
          (String) call.argument("stacktrace"), call.argument("platform"));
    } else if (call.method.equals("reportErrorInAction")) {
      reportErrorInAction((int) call.argument("key"), (String) call.argument("errorName"),
          (int) call.argument("errorCode"), call.argument("platform"));
    } else if (call.method.equals("identifyUser")) {
      identifyUser((String) call.argument("user"), call.argument("platform"));
    } else if (call.method.equals("reportEventInAction")) {
      reportEventInAction((int) call.argument("key"), (String) call.argument("name"), call.argument("platform"));
    } else if (call.method.equals("reportStringValueInAction")) {
      reportStringValueInAction((int) call.argument("key"), (String) call.argument("name"),
          (String) call.argument("value"), call.argument("platform"));
    } else if (call.method.equals("reportIntValueInAction")) {
      reportIntValueInAction((int) call.argument("key"), (String) call.argument("name"), (int) call.argument("value"),
          call.argument("platform"));
    } else if (call.method.equals("reportDoubleValueInAction")) {
      reportDoubleValueInAction((int) call.argument("key"), (String) call.argument("name"),
          (double) call.argument("value"), call.argument("platform"));
    } else if (call.method.equals("setGPSLocation")) {
      setGPSLocation((double) call.argument("latitude"), (double) call.argument("longitude"),
          call.argument("platform"));
    } else if (call.method.equals("flushEvents")) {
      flushEvents(call.argument("platform"));
    } else if (call.method.equals("isCrashReportingOptedIn")) {
      result.success(isCrashReportingOptedIn(call.argument("platform")));
    } else if (call.method.equals("setCrashReportingOptedIn")) {
      setCrashReportingOptedIn((boolean) call.argument("crashReporting"), call.argument("platform"));
    } else if (call.method.equals("setDataCollectionLevel")) {
      setDataCollectionLevel((int) call.argument("dataCollectionLevel"), call.argument("platform"));
    } else if (call.method.equals("getDataCollectionLevel")) {
      result.success(getDataCollectionLevel(call.argument("platform")));
    } else if (call.method.equals("applyUserPrivacyOptions")) {
      applyUserPrivacyOptions((int) call.argument("dataCollectionLevel"), (boolean) call.argument("crashReportingOptedIn"),
              call.argument("platform"));
    } else if (call.method.equals("getUserPrivacyOptions")) {
      result.success(getUserPrivacyOptions(call.argument("platform")));
    } else if (call.method.equals("getRequestTag")) {
      result.success(getRequestTag((int) call.argument("key"), (String) call.argument("url")));
    } else if (call.method.equals("getRequestTagForInterceptor")) {
      result.success(getRequestTagForInterceptor());
    } else if (call.method.equals("startWebRequestTiming")) {
      startWebRequestTiming((String) call.argument("requestTag"));
    } else if (call.method.equals("stopWebRequestTiming")) {
      stopWebRequestTiming((String) call.argument("requestTag"), (String) call.argument("url"),
          (int) call.argument("responseCode"), (String) call.argument("responseMessage"));
    } else if (call.method.equals("start")) {
      start((String) call.argument("beaconUrl"), (String) call.argument("applicationId"),
          (Boolean) call.argument("userOptIn"), (Boolean) call.argument("crashReporting"),
          (String) call.argument("logLevel"), (Boolean) call.argument("certificateValidation"));
    } else if (call.method.equals("getAutoStartConfiguration")) {
      HashMap<String, Boolean> configuration = new HashMap<>();
      configuration.put("autoStart", ConfigurationFactory.getConfiguration().autoStart);
      configuration.put("logLevel", ConfigurationFactory.getConfiguration().debugLogLevel);
      configuration.put("webRequest", ConfigurationFactory.getConfiguration().webRequestTiming);
      configuration.put("crashReporting", ConfigurationFactory.getConfiguration().crashReporting);
      result.success(configuration);
    } else {
      result.notImplemented();
    }
  }

  private void start(String beaconUrl, String applicationId, Boolean userOptIn, Boolean crashReporting, String logLevel,
      Boolean certificateValidation) {
    DynatraceConfigurationBuilder builder = new DynatraceConfigurationBuilder(applicationId, beaconUrl);

    if (userOptIn != null) {
      builder.withUserOptIn(userOptIn);
    }

    if (crashReporting != null) {
      builder.withCrashReporting(crashReporting);
    }

    if (logLevel != null && logLevel.equals("debug")) {
      builder.withDebugLogging(true);
    }

    if (certificateValidation != null) {
      builder.withCertificateValidation(certificateValidation);
    }

    Dynatrace.startup(applicationContext, builder.buildConfiguration());
  }

  private void enterAction(String name, int key, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      DTXAction action = Dynatrace.enterAction(name);
      actions.put(key, action);
      rootActions.add(key);
    }
  }

  private void enterActionWithParent(String name, int key, int parentKey, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      DTXAction parent = actions.get(parentKey);

      if (parent != null) {
        DTXAction action = Dynatrace.enterAction(name, parent);
        actions.put(key, action);
      } else {
        enterAction(name, key, platform);
      }
    }
  }

  private void leaveAction(int key) {
    DTXAction action = actions.get(key);

    if (action != null) {
      if (action instanceof DTXAutoAction) {
        ((DTXAutoAction) action).startTimer();
      } else {
        action.leaveAction();
      }

      actions.remove(key);
    }

    rootActions.remove((Integer) key);
  }

  private void endVisit(Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Dynatrace.endVisit();
    }
  }

  private void reportError(String errorName, int errorCode, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Dynatrace.reportError(errorName, errorCode);
    }
  }

  /**
   * Reports a stacktrace - Used primary for the internal error handler
   * 
   * @param errorName  Non or Fatal React Native Error
   * @param errorValue Name of the Error - SyntaxError
   * @param reason     Reason for the Error
   * @param stacktrace Whole Stacktrace
   * @param platform   Platform wise or both
   */
  private void reportErrorWithStacktrace(String errorName, String errorValue, String reason, String stacktrace,
      Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Dynatrace.reportError(PlatformType.DART, errorName, errorValue, reason, stacktrace);
    }
  }

  public void reportCrash(String errorValue, String reason, String stacktrace, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Dynatrace.reportCrash(PlatformType.DART, errorValue, reason, stacktrace);
      Dynatrace.createNewSession();
    }
  }

  private void reportErrorInAction(int key, String errorName, int errorCode, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      DTXAction action = actions.get(key);
      if (action == null)
        return;
      action.reportError(errorName, errorCode);
    }
  }

  private void identifyUser(String user, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Dynatrace.identifyUser(user);
    }
  }

  private void reportEventInAction(int key, String name, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      DTXAction action = actions.get(key);
      if (action == null)
        return;
      action.reportEvent(name);
    }
  }

  private void reportStringValueInAction(int key, String name, String value, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      DTXAction action = actions.get(key);
      if (action == null)
        return;
      action.reportValue(name, value);
    }
  }

  private void reportIntValueInAction(int key, String name, int value, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      DTXAction action = actions.get(key);
      if (action == null)
        return;
      action.reportValue(name, value);
    }
  }

  private void reportDoubleValueInAction(int key, String name, double value, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      DTXAction action = actions.get(key);
      if (action == null)
        return;
      action.reportValue(name, value);
    }
  }

  private void setGPSLocation(double latitude, double longitude, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Location location = new Location("");
      location.setLatitude(latitude);
      location.setLongitude(longitude);
      Dynatrace.setGpsLocation(location);
    }
  }

  private void flushEvents(Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Dynatrace.flushEvents();
    }
  }

  @Deprecated
  private boolean isCrashReportingOptedIn(Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      return Dynatrace.isCrashReportingOptedIn();
    } else {
      return false;
    }
  }

  @Deprecated
  private void setCrashReportingOptedIn(boolean crashReporting, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Dynatrace.setCrashReportingOptedIn(crashReporting);
    }
  }

  @Deprecated
  private void setDataCollectionLevel(int collectionLevel, Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      Dynatrace.setDataCollectionLevel(DataCollectionLevel.values()[collectionLevel]);
    }
  }

  @Deprecated
  private int getDataCollectionLevel(Object platform) {
    if (this.shouldWorkOnAndroid(platform)) {
      return Dynatrace.getDataCollectionLevel().ordinal();
    } else {
      return DATA_COLLECTION_OFF;
    }
  }

  private String getRequestTag(int key, String url) {
    if (key != 0 && url != null) {
      DTXAction action = actions.get(key);
      if (action != null) {
        return action.getRequestTag();
      }  
    }
    return Dynatrace.getRequestTag();
  }

  private String getRequestTagForInterceptor() {
    if (!rootActions.isEmpty()) {
      DTXAction action = actions.get(rootActions.getFirst());

      if (action != null) {
        return action.getRequestTag();
      }
    }
    return Dynatrace.getRequestTag();
  }

  private void startWebRequestTiming(String requestTag) {
    if (requestTag != null) {
      WebRequestTiming timing = Dynatrace.getWebRequestTiming(requestTag);
      if(timing != null){
        webTimings.put(requestTag, timing);
        timing.startWebRequestTiming();
      }
    }
  }

  private void stopWebRequestTiming(String requestTag, String url, int responseCode, String responseMessage) {
    if (requestTag != null) {
      WebRequestTiming timing = webTimings.get(requestTag);
      if (timing != null) {
        try {
          timing.stopWebRequestTiming(url, responseCode, responseMessage);
          webTimings.remove(requestTag);
        } catch (MalformedURLException ex) {
          // do nothing
        }
      }
    }
  }

  private void applyUserPrivacyOptions(int dataCollectionLevel, boolean crashReporting, Object platform){
    if(this.shouldWorkOnAndroid(platform)){
      UserPrivacyOptions.Builder optionsBuilder = UserPrivacyOptions.builder();
      optionsBuilder.withCrashReportingOptedIn(crashReporting);
      optionsBuilder.withDataCollectionLevel(DataCollectionLevel.values()[dataCollectionLevel]);

      Dynatrace.applyUserPrivacyOptions(optionsBuilder.build());
    }
  }

  private HashMap<String, Object> getUserPrivacyOptions(Object platform){
    HashMap<String, Object> optionsMap = new HashMap<>();

    if(this.shouldWorkOnAndroid(platform)){
        UserPrivacyOptions options = Dynatrace.getUserPrivacyOptions();
        optionsMap.put("dataCollectionLevel", options.getDataCollectionLevel().ordinal());
        optionsMap.put("crashReportingOptedIn", Boolean.valueOf(options.isCrashReportingOptedIn()));
    }

    return optionsMap;
  }

  private Boolean shouldWorkOnAndroid(Object platform) {
    return platform == null || ((int) platform) == PLATFORM_ANDROID;
  }
}
